--- EditSpellsState: class
-- Allows player to edit their current spells
EditSpellsState = class('EditSpellsState', GameState)

function EditSpellsState:init(gameplay)
    GameState.init(self)
    
    self.gameplay = gameplay
    self.drawPreviousState = true
    self.updatePreviousState = true
    --self.touchPreviousState = true
    self.scene.camera:get(camera).clearMode = 0
    
    self.gameplay.player.cameraOffset.y = -2
    
    self:setupUI()
end

function EditSpellsState:added(game)
end

function EditSpellsState:removed(game)
end

function EditSpellsState:setupUI()
    local scn = self.scene
    local canvas = scn.canvas
    
    local titleText = canvas:child()
    titleText.y = -50
    titleText.size = vec2(600, 100)
    titleText.anchor = vec2(0.5, 0.0)
    local titleTextLabel = titleText:add(ui.label)
    titleTextLabel.text = "Edit Spells"
    
    local scn = self.scene
    
    local hand = scn.canvas:child()
    hand.size = vec2(WIDTH - 150, 200)
    hand:anchor(STRETCH, BOTTOM)
    hand.pivot = vec2(0.5, 0)
    hand:add(ui.hstack)
    hand.sprite = ui.resources.space
    hand.y = 30
    
    local spell = scn.canvas:child()
    spell.size = vec2(WIDTH - 150, 130)
    spell:anchor(STRETCH, BOTTOM)
    spell.pivot = vec2(0.5, 0)
    spell:add(ui.hstack)
    spell.sprite = ui.resources.space
    spell.y = 30 + 200 + 10
    
    local gameplay = self.gameplay
    
    function updateCards()
        local cards = {}
        for k,v in pairs(spell.children) do
            if v.valid then
                table.insert(cards, v:get(MiniCardUI).card)
            end
        end
        gameplay.player.items[1]:setCards(cards)
    end
    
    for _, card in pairs(self.gameplay.player.cards) do  
        cardEnt = hand:child()
        cardEnt:add(CardUI, card)        
        function cardEnt:tapped(touch)
            self:destroy()            
            spellEnt = spell:child()
            spellEnt:add(MiniCardUI, self:get(CardUI).card)
            
            function spellEnt:tapped(touch)
                cardEnt = hand:child()
                cardEnt:add(CardUI, self:get(CardUI).card)                        
                self:destroy()
            end            
            updateCards()
        end
        function cardEnt:destroyed()
            updateCards()
        end
    end        
end

CardUI = class("CardUI")

function CardUI:created(card)
    self.card = card
    self.entity.size = vec2(140, 180)
    self.entity.sprite = ui.resources.space -- TODO: card sprite...
    self.entity.hitTest = true
end

function CardUI:draw()
    style.push()
        .fill(255)
        .textAlign(CENTER|MIDDLE)
        .font("ArialRoundedMTBold")
        .fontSize(20)
        .stroke(0)
        .strokeWidth(4)
    
    --rect(0, 0, 50, 50)
    local size = self.entity.size    
    text(self.card.title, size.x/2, size.y/2 - 20)    
    style.pop()
end

function CardUI:touched(touch)
    if touch.began and self.touch == nil then 
        self.touch = touch
        return true
    elseif touch.ended and self.touch.id == touch.id then
        if touch.tapCount == 1 then
            self.entity:dispatch("tapped", touch)
        end
        self.touch = nil
        return true
    elseif touch.cancelled then
        self.touch = nil
        return false
    end 
end

MiniCardUI = class("MiniCardUI", CardUI)

function MiniCardUI:created(card)
    self.card = card
    self.entity.size = vec2(90, 90)
    self.entity.sprite = ui.resources.space -- TODO: card sprite...
    self.entity.hitTest = true
end

function MiniCardUI:draw()
    style.push()
    .fill(255)
    .textAlign(CENTER|MIDDLE)
    .font("ArialRoundedMTBold")
    .fontSize(15)
    .stroke(0)
    .strokeWidth(2)
    
    local size = self.entity.size    
    text(self.card.title, size.x/2, size.y/2)    
    style.pop()
end
