require 'GameState'

-- Tilemap editor class to for map customisation
Editor = class("Editor", GameState)

ACTION_TILE = 1
ACTION_OBJECT = 2

TOOL_PENCIL = 1
TOOL_ERASE = 2
TOOL_MOVE = 3
TOOL_SAMPLE = 4

MAP_SIZE = 100     

function Editor:action(touch)
    local x, y = self.cam:screenToWorld(touch.x, touch.y)
    local tx, ty = self.layers[LayerID]:worldToTile(x, y)
    
    if Tool == TOOL_PENCIL then 
        if Action == ACTION_TILE then
            self:paint(tx, ty)    
        elseif Action == ACTION_OBJECT then
            self:place(Object.class, tx, ty, table.unpack(Object.args))
        end
    elseif Tool == TOOL_ERASE then            
        if EraseLayerID == nil then
            for i = #self.layers, 1, -1 do
                if self.layers[i]:get(tx, ty) ~= 0 then
                    EraseLayerID = i
                    break
                end 
            end            
        end
        if EraseLayerID then
            self.layers[EraseLayerID]:set(tx, ty, 0)
        end
        if touch.ended or touch.cancelled then EraseLayerID = nil end
    end    
end

function Editor:place(entityClass, tx, ty, ...)
    local ent = self.scene:entity(entityClass.name)
    ent.x, ent.y = self.layer1:tileToWorld(tx, ty)
    ent:add(entityClass, ...)        
    return ent
end

function Editor:paint(tx, ty)
    self.layers[LayerID]:set(tx, ty, TileID)        
end

function Editor:added(game)
    self.canvasRig.enabled = true
end

function Editor:removed(game)
    self.canvasRig.enabled = false
end

function Editor:init(key) 
    GameState.init(self)
    
    self.key = key    
    --self.scene.physics2d.debugDraw = true
    
    self.map = tm.tilemap.read(key)
    
    if self.map == nil then 
        self.map = tm.tilemap(tileset)    
        
        self.layer1 = self.map:layer("ground")                
        
        self.layer1:resize(MAP_SIZE, MAP_SIZE)
        self.layer1:fill(tileset.ground)
        
        self.layer2 = self.map:layer("details")
        self.layer3 = self.map:layer("objects")
        self.layer3:set(MAP_SIZE/2, MAP_SIZE/2, tileset.player.id)
        
        local wall = tileset.wallFront.id
        
        for i = 0, MAP_SIZE-1 do
            self.layer1:set(0, i, wall)
            self.layer1:set(MAP_SIZE-1, i, wall)
            self.layer1:set(i, 0, wall)
            self.layer1:set(i, MAP_SIZE-1, wall)
        end
        
    else 
        self.layer1 = self.map.ground
        self.layer2 = self.map.details
        self.layer3 = self.map.objects
    end
        
    self.map.world2d = self.scene.world2d
        
    self.mapEntity = self.scene:entity()    
    
    function self.mapEntity.draw(entity)
        self.map:draw()
    end
            
    function self.mapEntity.touched(entity, touch)
        if touch.began and self.id then
            self.id = nil
            return false
        end
        if touch.began and self.id == nil then
            self.id = touch.id
            self.pos = touch.pos
            return true  
        end
        if touch.moving and self.id == touch.id then     
            if self.pos:distance(touch.pos) > 15 then
                self:action(touch)
            end
        end 
        if (touch.ended or touch.cancelled) and self.id == touch.id then
            if touch.tapCount == 1 then 
                self:action(touch)
            end
            self.id = nil
        end
    end
    
    self.layers = 
    {
        self.layer1,
        self.layer2,
        self.layer3
    }
    
    -- Grab a reference to the scene's camera and center it over the tiles    
    self.canvasRig = self.scene.camera:add(camera.rigs.canvas, 10)    
    self.scene.camera.x = 4
    self.scene.camera.y = 4
    
    self.canvas = self.scene.canvas
        
    self.tileTools = self.canvas:child()
    self.tileTools:anchor(CENTER, TOP)
    self.tileTools.pivot = vec2(0.5, 1)
    self.tileTools.size = vec2(80, 80)
    self.mainToolbar = self.tileTools:add(ToolBar, self.canvas, false)
    self.tileTools.y = -30
    
    self.mainToolbar:addTileCategory
    {
        {icon = tileset.ground:sprite(), tile = tileset.ground},
        {icon = tileset.wall:sprite(), tile = tileset.wall},
        {icon = tileset.wallFront:sprite(), tile = tileset.wallFront}        
    }
    
    self.mainToolbar:addTileCategory
    {
        {icon = tileset.player:sprite(), tile = tileset.player, layer = 3},
    }

    self.actionTools = self.canvas:child()
    self.actionTools:anchor(RIGHT, MIDDLE)
    self.actionTools.pivot = vec2(1.0, 0.5)
    self.actionTools.size = vec2(80, 80)
    self.actionTools:add(ToolBar, self.canvas, true)
    
    self.actionTools:get(ToolBar):addTool({{icon = asset.pencil}}, function()
        Tool = TOOL_PENCIL
    end)    

    self.actionTools:get(ToolBar):addTool({{icon = asset.eraser}}, function()
        Tool = TOOL_ERASE
    end)    
    
    self.actionTools:get(ToolBar):addTool({{icon = asset.hand_move_tool}}, function()
        Tool = TOOL_MOVE
    end)    
    
    self.actionTools:get(ToolBar):addTool({{icon = asset.eye_dropper}}, function()
        Tool = TOOL_SAMPLE
    end)    

    self.actionTools:get(ToolBar):addTool({{icon = asset.media_player_ui_button_play}}, function()
        --tm.tileset.save(self.key, tileset)
        --tm.tilemap.save(self.key, self.map)
        local gameplay = Gameplay(self.map:copy())
        gameplay:addBackButton()
        self.game:push(gameplay)
        self.game:push(EditSpellsState(gameplay))
    end)    
    
    self.actionTools:get(ToolBar):addTool({{icon = asset.save}}, function()
        tm.tileset.save(self.key, tileset)
        tm.tilemap.save(self.key, self.map)
    end)    
    
    Tool = TOOL_PENCIL
    LayerID = 1
end


