-- Small class to manage basic stats for weapons, the player and monsters
Stat = class("Stat")

function Stat:init(name, base, parent)
    self.name = name
    self.base = base
    self.bonus = 0
    self.parent = parent
end

function Stat:value()
    local parent = self.parent and self.parent:value() or 0
    return self.base + self.bonus + parent
end

function Stat:reset()
    self.bonus = 0
end

function Stat:augment(amount)
    self.bonus = self.bonus + amount
end

Item = class("Item")

function Item:init(player)
    self.player = player
    self.level = 0
    --self.maxLevel = #self.class.UPGRADES
    --self:upgrade()
end

function Item:upgrade()
    local upgrades = self.class.UPGRADES
    if upgrades and upgrades[self.level+1] then
        self.level = self.level + 1
        if upgrades[self.level].upgrade then
            upgrades[self.level].upgrade(self) 
        end
    end
end

function Item:draw() end
function Item:update(dt) end

HealthPotion = class("HealthPotion")
HealthPotion.static.CONSUMABLE = true
HealthPotion.static.NAME = "Health Potion"
HealthPotion.static.DESCRIPTION = "Heal some health!"

function HealthPotion:init()
    self.heal = 25
end
    
function HealthPotion:consume(player) 
    player.mob:heal(self.heal)
end


ProteinPowder = class("ProteinPowder", Item)

ProteinPowder.static.NAME = "Protein Powder"
ProteinPowder.static.PASSIVE = true
ProteinPowder.static.UPGRADES =
{
    {
        text = "Increases damage inflicted by 10%",
        upgrade = function(item) item.player.stats.strength:augment(0.1) end
    },
    {
        text = "Raises damage by 10%", 
        upgrade = function(item) item.player.stats.strength:augment(0.1) end
    },
    {
        text = "Raises damage by 10%", 
        upgrade = function(item) item.player.stats.strength:augment(0.1) end
    },
    {
        text = "Raises damage by 10%", 
        upgrade = function(item) item.player.stats.strength:augment(0.1) end
    },    
    {
        text = "Raises damage by 10%", 
        upgrade = function(item) item.player.stats.strength:augment(0.1) end
    },
}

Vacuum = class("Vacuum", Item)

Vacuum.static.NAME = "Vacuum"
Vacuum.static.PASSIVE = true
Vacuum.static.UPGRADES =
{
    {
        text = "Increases vacuum range by 20%",
        upgrade = function(item) item.player.stats.vacuum:augment(0.2) end
    },
    {
        text = "Increases vacuum range by 20%",
        upgrade = function(item) item.player.stats.vacuum:augment(0.2) end
    },
    {
        text = "Increases vacuum range by 20%",
        upgrade = function(item) item.player.stats.vacuum:augment(0.2) end
    },
    {
        text = "Increases vacuum range by 20%",
        upgrade = function(item) item.player.stats.vacuum:augment(0.2) end
    },
    {
        text = "Increases vacuum range by 20%",
        upgrade = function(item) item.player.stats.vacuum:augment(0.2) end
    },
    {
        text = "Increases vacuum range by 20%",
        upgrade = function(item) item.player.stats.vacuum:augment(0.2) end
    },
    {
        text = "Increases vacuum range by 20%",
        upgrade = function(item) item.player.stats.vacuum:augment(0.2) end
    },
}

Haste = class("Haste", Item)

Haste.static.NAME = "Haste"
Haste.static.PASSIVE = true
Haste.static.UPGRADES =
{
    {
        text = "Increases attack speed by 10%",
        upgrade = function(item) item.player.stats.haste:augment(0.1) end
    },
    {
        text = "Increases attack speed by 10%",
        upgrade = function(item) item.player.stats.haste:augment(0.1) end
    },
    {
        text = "Increases attack speed by 10%",
        upgrade = function(item) item.player.stats.haste:augment(0.1) end
    },
    {
        text = "Increases attack speed by 10%",
        upgrade = function(item) item.player.stats.haste:augment(0.1) end
    },
    {
        text = "Increases attack speed by 10%",
        upgrade = function(item) item.player.stats.haste:augment(0.1) end
    },
}

Duper = class("Duper", Item)

Duper.static.NAME = "Duper"
Duper.static.PASSIVE = true
Duper.static.UPGRADES =
{
    {
        text = "Increases amount of projectiles by one",
        upgrade = function(item) item.player.stats.amount:augment(1) end
    },
    {
        text = "Increases amount of projectiles by one",
        upgrade = function(item) item.player.stats.amount:augment(1) end
    },
}

SpeedBoots = class("SpeedBoots", Item)

SpeedBoots.static.NAME = "Speed Boots"
SpeedBoots.static.PASSIVE = true
SpeedBoots.static.UPGRADES =
{
    {
        text = "Increases move speed by 0.2",
        upgrade = function(item) item.player.stats.moveSpeed:augment(0.2) end
    },
    {
        text = "Increases move speed by 0.2",
        upgrade = function(item) item.player.stats.moveSpeed:augment(0.2) end
    },
    {
        text = "Increases move speed by 0.2",
        upgrade = function(item) item.player.stats.moveSpeed:augment(0.2) end
    },
    {
        text = "Increases move speed by 0.2",
        upgrade = function(item) item.player.stats.moveSpeed:augment(0.2) end
    },
    {
        text = "Increases move speed by 0.2",
        upgrade = function(item) item.player.stats.moveSpeed:augment(0.2) end
    },
    {
        text = "Increases move speed by 0.2",
        upgrade = function(item) item.player.stats.moveSpeed:augment(0.2) end
    },
    {
        text = "Increases move speed by 0.2",
        upgrade = function(item) item.player.stats.moveSpeed:augment(0.2) end
    },
    {
        text = "Increases move speed by 0.2",
        upgrade = function(item) item.player.stats.moveSpeed:augment(0.2) end
    },
}

Weapon = class("Weapon", Item)

function Weapon:init(player)
    Item.init(self, player)
    self.player = player
    self.timer = 0
    
    self.stats = {}
    self.stats.cooldown = Stat("Cooldown", 2)
    self.stats.amount = Stat("Amount", 0, player.stats.amount)
    self.stats.baseDamage = Stat("Base Damage", 5)
    self.stats.pierce = Stat("Pierce", 1)
    self.stats.duration = Stat("Duration", 1)
    self.stats.knockback = Stat("Knockback", 2)
    self.stats.area = Stat("Area", 0)
    self.stats.speed = Stat("Speed", 1)
    self.stats.repeats = Stat("Repeats", 0)
end

function Weapon:draw() end

function Weapon:update(dt)
    Item.update(self, dt)
    self.timer = math.max(self.timer - dt * (1 + self.player.stats.haste:value()), 0)
    if self.timer == 0 then
        self:fire()
    end
end

function Weapon:fire()
    self.timer = self.stats.cooldown:value()
end

function Weapon:upgrade()
    Item.upgrade(self)
    self.timer = 0
end

-- spawn a projectile
-- args{sprite, sx, sy, radius, gravityScale, life, pierce, follow, orbit, knockback, damage}
function Weapon:spawnProjectile(x, y, r, vx, vy, av, args)
    local proj = self.player.gameplay.scene:entity()
    proj.x = x
    proj.y = y
    proj.rz = r
    proj.sprite = args.sprite
    proj:destroy(args.life or 1)
    local body = proj:add(physics2d.body, DYNAMIC)
    body.linearVelocity = vec2(vx, vy)
    body.angularVelocity = av
    body.gravityScale = args.gravityScale or 1
    local collider = args.size and body:box(args.size.x, args.size.y) or body:circle(args.radius or 0.5)
    collider.category = WEAPON_CAT
    collider.mask = ENEMY_CAT
    collider.sensor = true
    local player = self.player
    local damage = args.damage * (1 + player.stats.strength:value())
    local pierce = args.pierce or -1
    local follow = args.follow or false
    local orbit = args.orbit
    local knockback = args.knockback or 0
    
    if follow or orbit then
        proj.x = player.entity.x - 100
        proj.y = player.entity.y - 100
        body.position = vec2(proj.x, proj.y)
        function proj:update(dt) 
            if orbit then
                local a = math.rad(orbit) * dt
                local x2 = math.cos(a) * x - math.sin(a) * y
                local y2 = math.sin(a) * x + math.cos(a) * y
                x, y = x2, y2
            end
            proj.x = player.entity.x + x
            proj.y = player.entity.y + y
            body.position = vec2(proj.x, proj.y)            
        end
    end
    
    function proj:collisionBegan2d(collision)
        local mob = collision.otherBody.entity and collision.otherBody.entity:get(Mob)
        if mob and pierce ~= 0 then
            mob:applyDamage(player, damage)
            mob:applyKnockback(player.entity.x, player.entity.y, knockback)
            pierce = pierce - 1
            if pierce == 0 then
                proj:destroy(0.01)
                return
            end
        end
    end
    
    return proj
end

function Weapon:damageBox(sx, sy, x, y, w, h, amount, knockback)
    amount = amount + (1 + self.player.stats.strength:value())
    local results = self.world2d:queryBoxAll(x, y, w, h, ENEMY_CAT)
    for k,v in pairs(results) do
        local mob = v.body.entity:get(Mob)
        if mob and mob ~= self.player.mob then
            mob:applyDamage(self.player, amount)
            mob:applyKnockback(sx, sy, knockback)
        end
    end
end

Whip = class("Whip", Weapon)

Whip.static.NAME = "Sword"
Whip.static.UPGRADES =
{
    {
        text = "Sword!",
    },
    {
        text = "Increase projectile count by one", 
        upgrade = function(axe) axe.stats.amount:augment(1) end
    },
    {
        text = "Increase damage by 20. Increase area",
        upgrade = function(axe) 
            axe.stats.baseDamage:augment(20) 
            axe.stats.area:augment(0.25)
        end
    },
    {
        text = "Reduce cooldown. Increase duration",
        upgrade = function(axe) 
            axe.stats.cooldown:augment(-0.25) 
            axe.stats.duration:augment(0.25) 
        end
    },
    {
        text = "Increase damage by 40",
        upgrade = function(axe) axe.stats.baseDamage:augment(40) end
    },
    {
        text = "Reduce cooldown. Increase duration",
        upgrade = function(axe) 
            axe.stats.cooldown:augment(-0.25) 
            axe.stats.duration:augment(0.25) 
        end
    },
}

function Whip:init(player)
    Weapon.init(self, player)
    
    self.stats.baseDamage.base = 5
    self.stats.cooldown.base = 1.5
    self.stats.duration.base = 0.5
    self.stats.knockback.base = 3
    
    self.flip = false
    
    self.sprite = tileset.sword:sprite()
    self.icon = self.sprite
    self.world2d = self.player.gameplay.scene.world2d
    
    self.projectile =
    {
        sprite = self.sprite,
        size = {1, 2},
        gravityScale = 0,
        follow = true,
        pierce = -1,        
    }
end

function Whip:fire()
    Weapon.fire(self)
    
    local x, y = self.player.entity.x, self.player.entity.y
    local dir = self.flip and -1 or 1
    
    self.projectile.damage = self.stats.baseDamage:value()
    self.projectile.knockback = self.stats.knockback:value()
    self.projectile.life = self.stats.duration:value()
    local area = self.stats.area:value()
    self.projectile.size[1] = 1 + area
    self.projectile.size[2] = 2 + area
    
    local amount = self.stats.amount:value()
    for i = 0, amount do
        if i == 1 then dir = dir * -1 end
        self:spawnProjectile(dir, 0.25 + math.random() * 0.025, -90 * dir, 0, 0, 0, self.projectile)
    end  
    
    self.flip = not self.flip
end
    
function Whip:update(dt)
    Weapon.update(self, dt)
end

function Whip:draw()        
end

Axe = class("Axe", Weapon)

Axe.static.NAME = "Axe"
Axe.static.UPGRADES =
{
    -- 1 
    {
        text = "And my axe!",
    },
    -- 2
    {
        text = "Increase projectile count by one", 
        upgrade = function(axe) axe.stats.amount:augment(1) end
    },
    -- 3
    {
        text = "Increase damage by 20",
        upgrade = function(axe) axe.stats.baseDamage:augment(20) end
    },
    -- 4
    {
        text = "Pass through 2 more enemies",
        upgrade = function(axe) axe.stats.pierce:augment(2) end
    },
    -- 5
    {
        text = "Increase projectile count by one", 
        upgrade = function(axe) axe.stats.amount:augment(1) end
    },
    -- 6
    {
        text = "Increase damage by 20",
        upgrade = function(axe) axe.stats.baseDamage:augment(20) end
    },
    -- 7
    {
        text = "Pass through 2 more enemies",
        upgrade = function(axe) axe.stats.pierce:augment(2) end
    },
    -- 8
    {
        text = "Increase damage by 20",
        upgrade = function(axe) axe.stats.baseDamage:augment(20) end
    }, 
}

function Axe:init(player)
    Weapon.init(self, player)
    self.sprite = tileset.axe:sprite()
    self.icon = self.sprite
    self.world2d = self.player.gameplay.scene.world2d
    self.maxLevel = 8
    
    self.stats.baseDamage.base = 10    
    self.stats.knockback.base = 5
    self.stats.cooldown.base = 3
    
    self.projectile =
    {
        sprite = self.sprite,
        radius = 0.5,
        gravityScale = 1,
        life = 2,
    }
end

function Axe:fire()
    Weapon.fire(self)
    
    local amount = self.stats.amount:value()
    local dir = self.player.mob.facing.x >= 0 and 1 or -1
    
    self.projectile.damage = self.stats.baseDamage:value()
    self.projectile.knockback = self.stats.knockback:value()
    self.projectile.pierce = self.stats.pierce:value()
    
    for i = 0, amount do
        self:spawnProjectile(self.player.entity.x, self.player.entity.y, 0, 
            i * 3 * dir, 6, 360,
            self.projectile)
    end
end

Hammer = class("Hammer", Weapon)

Hammer.static.NAME = "Hammer"
Hammer.static.UPGRADES =
{
    -- 1 
    {
        text = "A mighty hammer!",
    },
    -- 2
    {
        text = "Increase projectile count by one", 
        upgrade = function(axe) axe.stats.amount:augment(1) end
    },
    -- 3
    {
        text = "Increase damage by 5",
        upgrade = function(axe) axe.stats.baseDamage:augment(5) end
    },
    -- 4
    {
        text = "Increase projectile count by one. Reduce cooldown", 
        upgrade = function(axe) 
            axe.stats.amount:augment(1) 
            axe.stats.cooldown:augment(-1)
        end
    },
    -- 5
    {
        text = "Increase projectile count by one", 
        upgrade = function(axe) axe.stats.amount:augment(1) end
    },
    -- 6
    {
        text = "Increase damage by 10",
        upgrade = function(axe) axe.stats.baseDamage:augment(10) end
    },
    -- 7
    {
        text = "Increase projectile count by one", 
        upgrade = function(axe) axe.stats.amount:augment(1) end
    },
    -- 8
    {
        text = "Increase damage by 20",
        upgrade = function(axe) axe.stats.baseDamage:augment(40) end
    }, 
}

function Hammer:init(player)
    Weapon.init(self, player)
    self.sprite = tileset.hammer:sprite()
    self.icon = self.sprite
    self.world2d = self.player.gameplay.scene.world2d
    self.maxLevel = 8
    
    self.stats.baseDamage.base = 5    
    self.stats.knockback.base = 5
    self.stats.cooldown.base = 6
    
    self.projectile =
    {
        sprite = self.sprite,
        radius = 0.5,
        gravityScale = 1,
        life = 4,
        follow = true,
        orbit = 180,
        pierce = -1
    }    
end

function Hammer:fire()
    Weapon.fire(self)
    
    local amount = self.stats.amount:value()
    
    self.projectile.damage = self.stats.baseDamage:value()
    self.projectile.knockback = self.stats.knockback:value()
    
    local spacing = 2 * math.pi / (amount+1)
    for i = 0, amount do
        self:spawnProjectile(math.cos(spacing * i) * 2, math.sin(spacing * i) * 2, spacing * i,
            0, 0, 360,
            self.projectile)
    end
end

Knife = class("Knife", Weapon)

Knife.static.NAME = "Knife"
Knife.static.UPGRADES =
{
    -- 1 
    {
        text = "A throwing knife!",
    },
    -- 2
    {
        text = "Increase speed", 
        upgrade = function(axe) axe.stats.speed:augment(1) end
    },
    -- 3
    {
        text = "Increase damage by 10",
        upgrade = function(axe) axe.stats.baseDamage:augment(20) end
    },
    -- 4
    {
        text = "Decrease cooldown",
        upgrade = function(axe) axe.stats.cooldown:augment(0.1) end
    },
    -- 5
    {
        text = "Increase speed", 
        upgrade = function(axe) axe.stats.speed:augment(1) end
    },
    -- 6
    {
        text = "Increase damage by 20",
        upgrade = function(axe) axe.stats.baseDamage:augment(20) end
    },
    -- 7
    {
        text = "Decrease cooldown",
        upgrade = function(axe) axe.stats.cooldown:augment(0.1) end
    },
    -- 8
    {
        text = "Increase damage by 20",
        upgrade = function(axe) axe.stats.baseDamage:augment(20) end
    }, 
}

function Knife:init(player)
    Weapon.init(self, player)
    self.sprite = tileset.knife:sprite()
    self.icon = self.sprite
    self.world2d = self.player.gameplay.scene.world2d
    self.maxLevel = 8
    
    self.stats.baseDamage.base = 5
    self.stats.cooldown.base = 1
    self.stats.knockback.base = 2
    self.stats.speed.base = 2
    
    self.projectile =
    {
        sprite = self.sprite,
        radius = 0.25,
        gravityScale = 0,
        life = 2,
    }
end

function Knife:update(dt)
    Weapon.update(self, dt)   
end

function Knife:fire()
    Weapon.fire(self)
    
    local amount = self.stats.amount:value()
    
    self.projectile.damage = self.stats.baseDamage:value()
    self.projectile.knockback = self.stats.knockback:value()
    self.projectile.pierce = self.stats.pierce:value()
    
    local dir = self.player.mob.facing
    local speed = self.stats.speed:value()
    for i = 0, amount do
        self:spawnProjectile(self.player.entity.x, self.player.entity.y, 0, 
        dir.x * speed, dir.y * speed, -360,
        self.projectile)
    end
end
