--- LevelUpState: class
-- Displays available upgrades during level-up events
LevelUpState = class('LevelUpState', GameState)

function LevelUpState:init(gameplay)
    GameState.init(self)
    
    self.gameplay = gameplay
    self.drawPreviousState = true
    self.scene.camera:get(camera).clearMode = 0
    
    self:setupUI()
end

function LevelUpState:added(game)
end

function LevelUpState:removed(game)
end

function LevelUpState:setupUI()
    local scn = self.scene
    local canvas = scn.canvas
    
    local titleText = canvas:child()
    titleText.y = -50
    titleText.size = vec2(600, 100)
    titleText.anchor = vec2(0.5, 0.0)
    local titleTextLabel = titleText:add(ui.label)
    titleTextLabel.text = "Level Up"
    
    local scn = self.scene
    local buttons = scn.canvas:child()
    buttons.size = vec2(900, 600)
    buttons:anchor(CENTER, MIDDLE)
    buttons:add(ui.vstack)
    buttons.sprite = ui.resources.space
    
    titleText.parent = buttons
    
    local items = {}
    local itemMap = {}
    
    -- TODO: generate list of eligible items
    
    local eligible = {}
    for _, item in pairs(self.gameplay.items) do
        local level = self.gameplay.player:itemLevel(item.class) or 0       
        if item.class.CONSUMABLE or item.class.UPGRADES[level+1] then
            table.insert(eligible, item)
        end
    end
    
    local totalChance = 0
    for _, item in pairs(eligible) do
        totalChance = totalChance + item.chance
    end
    
    while #items < math.min(3, #eligible) do
        --local randomItem = eligible[math.random(1, #eligible)]
        local randomItem = eligible[1]
        local p = math.random()
        local c = 0
        for _, item in pairs(eligible) do
            if p < item.chance/totalChance+c then
                randomItem = item
                break
            end
            c = c + item.chance/totalChance
        end
        
        if itemMap[randomItem] == nil then
            itemMap[randomItem] = randomItem
            table.insert(items, randomItem)
        end
    end
        
    for _, item in pairs(items) do
        if item.class.CONSUMABLE then
            local str = string.format("%s: %s", item.class.NAME, item.class.DESCRIPTION)
            local itemButton = createButton(scn, str, 200, 75, ui.resources.grey)
            itemButton.parent = buttons
            
            local label = itemButton:get(ui.button).label
            label.fontSize = 18
            label.color = color.black
            
            function itemButton.tapped(entity, btn)
                item.class():consume(self.gameplay.player)
                self.game:pop()
            end                  
        else        
            local level = self.gameplay.player:itemLevel(item.class) or 0        
            local str = string.format("%s Level %d: %s", item.class.NAME, level+1, item.class.UPGRADES[level+1].text)
            local itemButton = createButton(scn, str, 200, 75, ui.resources.grey)
            itemButton.parent = buttons
            
            local label = itemButton:get(ui.button).label
            label.fontSize = 18
            label.color = color.black
            
            function itemButton.tapped(entity, btn)
                self.gameplay.player:equipItem(item.class)
                self.game:pop()
            end        
        end
        
    end
end
    