
-- Codea Survivors (2024)
-- Author: John Millard
-- 
-- A basic rogue-like survival game example inspired by games like Vampire Survivors
--
-- This example uses:
--   scene     - object management
--   physics2d - dynamic objects and collision checks
--   tilemap   - efficient map rendering/management
--
-- Contents:
-- GameState    - A management class for handling overall game state
-- Gameplay     - Gameplay state
-- LevelUpState - A simple menu to select player upgrades/items
-- Player       - A component for handling player logic
-- Monster      - A component for handling monster logic
-- Items        - A series of classes handling weapon/passive item logic
-- Mob          - A component for handling dynamic creature logic
-- Tiles        - Loading and setup for tileset atlas and tilemap
-- Data         - Data used to run the game
-- Editor       - A simple tilemap editor allowing for customisation
-- ToolBar      - Used for editor UI buttons
-- UI           - Some helper methods for creating UI style/buttons/etc...

viewer.mode = FULLSCREEN
-- viewer.displayStats = true
screenWidth, screenHeight = WIDTH, HEIGHT
function setup()
    colorspace(GAMMA)
    
    -- Get the tileset and atlas
    tileset, tilesetAtlas = createTileset()
    ui.resources = loadUI()
    loadData()
    
    
    greyBox = image.read(asset.builtin.UI.Grey_Box).slice:patch(10)    
        
    game = Game(Editor(asset.."/tilemap.map"))
end

function draw()
    if game then
        game:draw()
        game:update(time.delta)
    end
end

function sizeChanged(w, h)
    if game then
        game:sizeChanged(w, h)
    end
end

function touched(touch)
    game:touched(touch)
end
