--- Mob: class
-- Component for all mobile creatures (including the player)
Mob = class()

function Mob:created(gameplay, spr, category)
    self.sprite = spr
    self.gameplay = gameplay
    self.map = gameplay.map
    self.ground = self.map.ground
    self.objects = self.map.objects
    self.health = 100
    self.maxHealth = 100
    self.showHealth = true
    self.flash = 0
    self.shake = 0
    self.disableOnKill = false
    self.sturdyness = 0
    
    -- Create physics body/collider for simulating movement / handling collisions
    self.body = self.entity:add(physics2d.body, DYNAMIC)
    self.body.gravityScale = 0
    self.body.fixedRotation = true
    self.collider = self.body:circle(0.425, 0.0, 0.0)
    self.collider.restitution = 0.75
    self.collider.friction = 0
    self.collider.category = category 
    
    self.dir = vec2(0, 0)
    self.facing = vec2(0, 0)
    self.knockback = vec2()
end

function Mob:applyKnockback(x, y, knockback)
    local kx = self.entity.x - x
    local ky = self.entity.y - y
    local kl = math.sqrt(kx*kx + ky*ky) 
    self.knockback.x = self.knockback.x + kx/kl * knockback * (1-self.sturdyness)
    self.knockback.y = self.knockback.y + ky/kl * knockback * (1-self.sturdyness)
end

function Mob:destroyed()
    if self.health <= 0 then
        self.entity:dispatch("onKilled", self.deleted)
    end
end

function Mob:killed(dealer, deleted)    
    self.deleted = deleted
    if self.disableOnKill then
        self.entity.active = false
        self.body.linearVelocity = vec2(0, 0)
    else
        self.entity:destroy(0.25)
    end
end

function Mob:heal(amount)
    if self.health < self.maxHealth then
        self.health = math.min(self.health + amount, self.maxHealth)
    end
end

function Mob:applyDamage(dealer, amount, deleted)
    if self.health > 0 then        
        
        if amount == 0 then return end
                
        self.entity:dispatch("onDamaged", dealer, amount)
        self.gameplay.scene:tween(self):to{flash = 1}:time(0.1):to{flash = 0}:time(0.4)        
        if self.health - amount <= 0 then
            self.health = 0
            self:killed(dealer, deleted)
            self.shake = .2
        else            
            self.shake = .1
            self.health = math.max(self.health - amount, 0)            
        end
    end
end

function Mob:update(dt)
    if self.dir.x ~= 0 or self.dir.y ~= 0 then
        self.facing.x = self.dir.x
        self.facing.y = self.dir.y
    end
    self.body.linearVelocity = self.dir + self.knockback
    self.knockback.x = self.knockback.x * (1-dt*10)
    self.knockback.y = self.knockback.y * (1-dt*10)
end

function Mob:coords()
    return self.objects:worldToTile(self.entity.x, self.entity.y)
end
    
function Mob:draw()
    local tx, ty = self:coords()    
    style.push()
    
    style.tintMode(ADDITIVE).tint(255 * self.flash).sortOrder(ty - 1000)
    self.sprite:flip(self.facing.x < 0, false)
    sprite(self.sprite, 0, 0)    
    style.pop()
end
