--- Monster: class
Monster = class("Monster")

function Monster:created(gameplay, params)
    self.gameplay = gameplay   
    self.sprite = tileset[params.sprite]:sprite()
    self.sprite:anchor(params.ax or 0.5, params.ay or 0.5)
    self.entity.scale = vec3(1,1,1) * (params.scale or 1)
    self.mob = self.entity:add(Mob, gameplay, self.sprite, ENEMY_CAT)
    self.mob.collider.callbackMask = PLAYER_CAT
    self.mob.collider.radius = self.mob.collider.radius * (params.scale or 1)
    self.mob.collider.mask = params.mask or 0xFFFF
    self.body = self.mob.body
    self.hpPerLevel = params.hpPerLevel or 4
    self.mob.maxHealth = (params.health or 10) + self.hpPerLevel * gameplay.player.level
    self.mob.health = params.health or 10
    self.mob.sturdyness = params.sturdyness or 0
    self.dir = vec2(0,0)
    self.speed = params.speed or 1
    self.damage = params.damage or 100
    self.xp = params.xp or 5
    self.height = params.height or 1
    self.touchingPlayer = false
end

function Monster:onKilled(deleted)
    self.gameplay:onEnemyKilled(self)
    local player = self.gameplay.player
    if player then        
        player:onKilledMonster(self)
    end
    
    if not deleted then
        local g = self.gameplay.scene:entity()
        g.position = self.entity.position
        g:add(Gem, self.gameplay, tileset.potion:sprite(), self.xp)
    end
end

function Monster:onDamaged(dealer, amount)
    self.gameplay:showDamageNumber(self.entity.x, self.entity.y, amount)
end

function Monster:collisionBegan2d(collision)
    if collision.otherBody.entity == self.gameplay.playerEntity then
        self.touchingPlayer = true        
    end
end

function Monster:collisionEnded2d(collision)
    if collision.otherBody.entity == self.gameplay.playerEntity then
        self.touchingPlayer = false
    end
end

function Monster:update(dt)
    local player = self.gameplay.player
    if player then        
        if self.touchingPlayer then
            player.mob:applyDamage(self, self.damage * time.fixedDelta)
        end
        
        if self.swarmDir then
            self.mob.dir.x = self.swarmDir.x * self.speed
            self.mob.dir.y = self.swarmDir.y * self.speed
        else
            self.dir.x = player.entity.x - self.entity.x
            self.dir.y = player.entity.y - self.entity.y
            self.dir:normalize()
            self.mob.dir.x = self.dir.x * self.speed
            self.mob.dir.y = self.dir.y * self.speed
        end
    end
end

Gem = class("Gem")

function Gem:created(gameplay, spr, xp)
    self.gameplay = gameplay
    self.sprite = spr
    self.entity.sprite = spr
    self.xp = xp
    self.body = self.entity:add(physics2d.body, STATIC)
    self.collider = self.body:circle(0.35)
    self.collider.sensor = true
    self.collider.category = ITEM_CAT
    self.collider.mask = PLAYER_CAT
    self.collected = false
    self.vacuum = 0
end

function Gem:collect()
    if not self.collected then
        local player = self.gameplay.player
        self.collected = true
        self.startPos = self.entity.position
        self.gameplay.scene:tween(self)
            :to{vacuum = 1}
            :time(0.5)
            :ease(tween.backIn)
            :onComplete(function()
                player:grantXP(self.xp)
                self.entity:destroy()            
            end)
    end
end

function Gem:update(dt)
    if self.collected then
        self.entity.position = self.startPos * (1-self.vacuum) + self.gameplay.player.entity.position * self.vacuum
    end
end

function Gem:collisionBegan2d_(collision)
    local player = self.gameplay.player
    if player then
        player:grantXP(self.xp)
        self.entity:destroy()
    end
end
