--- createTileset Create and setup tilemap and tileset for the game
function createTileset()
    -- Step one - load our tileset asset from project assets
    -- to make things easy, Kenney's Tiny Dungeon© asset pack
    -- has been included

    local tilesetImg = image.read(asset["tileset.png"])
    -- It's pixel art, so disable smoothing
    tilesetImg.smooth = false    
    
    -- Grab the atlas for this image and slice it into 16x16 pieces
    local tilesetAtlas = tilesetImg.atlas
    tilesetAtlas.image.smooth = false
    tilesetAtlas:setWithCellSize(16, 16)
    atlas.save(tilesetAtlas)
    
    local tileset = tm.tileset()
    
    local groundRules = tm.ruleset()
    groundRules:rule()
    :sprite(tilesetAtlas.c5r5) -- corner shadow
    :rotate(true)
    :area(" xx",
          " @x",
          "   ")
    
    groundRules:rule()
    :sprite(tilesetAtlas.c4r5) -- wall
    :rotate(true)
    :area(" x ",
          " @ ",
          "   ")    
    
    groundRules:rule()
    :sprite(tilesetAtlas.c6r5) -- outer corner
    :rotate(true)
    :area("x= ",
          "=@ ",
          "   ")  
    
    groundRules:rule()
    :random(tilesetAtlas.c2r5, tilesetAtlas.c1r5, tilesetAtlas.c7r4)
    :rotate(true):flip(true,true)
    :area("   ",
          " @ ",
          "   ")       
    
    local ground = tileset:tile("ground")
    :sprite(tilesetAtlas.c2r5)
    :ruleset(groundRules)
    
    local wallRules = tm.ruleset()
    
    wallRules:rule():sprite(tilesetAtlas.c5r1)
    :flip(true, false)
    :area(" x ",
          "x@ ",
          "   ")
    
    wallRules:rule():sprite(tilesetAtlas.c5r2)
    :flip(true, false)
    :area("   ",
          "x@ ",
          " x ")
    
    wallRules:rule():sprite(tilesetAtlas.c4r2)
    :flip(true, false)
    :area("   ",
          "x@ ",
          "   ")
    
    wallRules
    :rule():sprite(tilesetAtlas.c3r3)
    :area(" x ",
          " @ ",
          "   ")
    
    wallRules:rule():sprite(tilesetAtlas.c3r1)
    :area("   ",
          " @ ",
          " x ")
    
    wallRules:rule():sprite(tilesetAtlas.c2r3)
    :flip(true, false)
    :area(" =x",
          " @=",
          "   ")
    
    wallRules:rule():sprite(tilesetAtlas.c4r1)
    :flip(true, false)
    :area("   ",
          "=@ ",
          "x= ")
    
    wallRules:rule():random(tilesetAtlas.c1r1, tilesetAtlas.c1r2, tilesetAtlas.c1r3)
    :area(" = ",
          "=@=",  
          " = ")
    
    local wall = tileset:tile("wall")
    :sprite(tilesetAtlas.c5r2)
    :group(1)
    :collision(tm.collision.square)
    :ruleset(wallRules)
    
    local wallFrontRules = tm.ruleset()
    
    wallFrontRules:rule():sprite(tilesetAtlas.c5r4)
    :flip(true, false)
    :area("   ",
          "g@g",
          "   ") 
    
    wallFrontRules:rule():sprite(tilesetAtlas.c10r5)
    :flip(true, false)
    :area("   ",
          "x@g",
          "   ")
    
    local wallFront = tileset:tile("wallFront")
    :sprite(tilesetAtlas.c5r4)
    :group(1)
    :collision(tm.collision.square)
    :ruleset(wallFrontRules)
    
    local doorRules = tm.ruleset()
    
    doorRules:rule()
    :sprite(tilesetAtlas.c12r4)
    :flip(true, false)
    :area("   ",
          "=@ ",
          "   ")
    
    local door = tileset:tile("door")
    :sprite(tilesetAtlas.c10r4)
    :group(1)
    :ruleset(doorRules)
    
    local trackRules = tm.ruleset()
    
    trackRules:rule():sprite(tilesetAtlas.c10r7)
    :area(" = ",
          " @ ",
          " = ")
    
    trackRules:rule():sprite(tilesetAtlas.c10r6)
        :flip(true, false)
        :area("   ",
              " @=",
              " = ")
    
    trackRules:rule():sprite(tilesetAtlas.c10r8)
    :flip(true, false)
    :area(" = ",
          " @=",
          "   ")
    
    trackRules:rule():sprite(tilesetAtlas.c11r6)
    :area("   ",
          "=@=",
          "   ")
    trackRules:rule():sprite(tilesetAtlas.c8r7)
    :flip(true, false)
    :area("   ",
          "x@=",
          "   ")        
    trackRules:rule():sprite(tilesetAtlas.c8r6)
        :flip(false, true)
        :area(" x ",
              " @ ",
              " = ")        
    
    local track = tileset:tile("track")
    :sprite(tilesetAtlas.c10r7)
    :group(1)
    :ruleset(trackRules)
    
    local spikes = tileset:tile("spikes")
    :sprite(tilesetAtlas.c6r4)

    local wizard = tileset:tile("wizard")
    :sprite(tilesetAtlas.c1r8)

    local knight = tileset:tile("knight")
    :sprite(tilesetAtlas.c1r9)

    local slime = tileset:tile("slime")
    :sprite(tilesetAtlas.c1r10)

    local cyclops = tileset:tile("cyclops")
    :sprite(tilesetAtlas.c2r10)

    local crab = tileset:tile("crab")
    :sprite(tilesetAtlas.c3r10)
    
    local bat = tileset:tile("bat")
    :sprite(tilesetAtlas.c1r11)
    
    local ghost = tileset:tile("ghost")
    :sprite(tilesetAtlas.c2r11)

    local spider = tileset:tile("spider")
    :sprite(tilesetAtlas.c3r11)
    
    local rat = tileset:tile("rat")
    :sprite(tilesetAtlas.c4r11)
    
    local chest = tileset:tile("chest")
    :sprite(tilesetAtlas.c6r8)

    local player = tileset:tile("player")
    :sprite(tilesetAtlas.c2r9)

    local potion = tileset:tile("potion")
    :sprite(tilesetAtlas.c9r10)

    local hammer = tileset:tile("hammer")
    :sprite(tilesetAtlas.c10r10)
    
    local axe = tileset:tile("axe")
    :sprite(tilesetAtlas.c11r10)
    
    local knife = tileset:tile("knife")
    :sprite(tilesetAtlas.c8r9)

    local sword = tileset:tile("sword")
    :sprite(tilesetAtlas.c11r9)
                
    local cart = tileset:tile("cart")
    :sprite(tilesetAtlas.c7r5)
    
    local itemSlot = tileset:tile("itemSlot")
    :sprite(tilesetAtlas.c1r6)
    
    tm.tileset.save(asset.."/tilemap", tileset)
    
    return tileset, tilesetAtlas
end
