ToolBar = class("ToolBar")

function ToolBar:addTileCategory(tiles)
    self:addTool(tiles, function(obj)
        TileID = obj.tile.id
        LayerID = obj.layer or 1
        Action = ACTION_TILE
    end)
end

function ToolBar:addObjectCategory(objects)
    self:addTool(objects, function(obj)        
        Object = obj
        Action = ACTION_OBJECT
    end)
end

function ToolBar:created(canvas, vertical)
    self.canvas = canvas
    self.entity.sprite = greyBox
    self.stack = self.entity:add(vertical and ui.vstack or ui.hstack)
    self.padding = 10
    
    self.active = self.canvas:child()
    function self.active:draw()
        style.push().fill(81, 220, 60).noStroke()
        rect(0, 0, 20, 5, 3)
        style.pop()
    end
    
    self.active:anchor(LEFT, BOTTOM)
    self.active.pivot = vec2(0.5, 0.5)
    self.active.size = vec2(20, 10)
    self.active.active = false
end

function ToolBar:toolSelected(tool)
    self.active.active = true
    self.active.position = tool.worldPosition
    self.active:translate(30, 10)
    
    tool:get(ToolButton):selected()
    for k,v in pairs(self.entity.children) do
        if v ~= tool then
            v:get(ToolButton):deselected()
        end
    end
end

function ToolBar:addTool(list, action)
    self.entity:child():add(ToolButton, self, list, action) 
end

ToolButton = class("TileButton")

function ToolButton:selected()
    self.button.selected = true
end

function ToolButton:deselected()
    self.button.selected = false
    self:close()
end

function ToolButton:close()
    if self.listEntity then
        self.listEntity.active = false
    end
end

function ToolButton:created(toolbar, list, func)
    self.entity.size = vec2(60, 60)
    self.toolbar = toolbar
    self.list = list
    
    -- Active Button
    local btn = self.entity:add(ui.button)
    self.button = btn
    btn.label.color = color(0)
    btn.label.text = ""
    btn.icon = list[1].icon
    
    function self.entity.tapped(entity, btn)     
        toolbar:toolSelected(entity)
        if #list == 1 then            
            if func then func(list[1]) end
        else 
            if self.listEntity.active then
                self.listEntity.active = false
            else 
                self.listEntity.active = true
            end        
        end
    end
    
    -- Additional Button List
    if #list > 1 then
        self.listEntity = self.toolbar.canvas:child()
        self.listEntity.sprite = greyBox
        self.listEntity.size = vec2(80, 80)
        self.listEntity:anchor(LEFT, BOTTOM)
        self.listEntity.pivot = vec2(0.5, 1)
        self.stack = self.listEntity:add(ui.vstack)
        
        for i = 1, #list do
            local listButton = self.listEntity:child()
            listButton.size = vec2(60, 60)
            local btn = listButton:add(ui.button)
            btn.label.text = ""
            btn.icon = list[i].icon
            
            function listButton.tapped(entity, btn)
                toolbar:toolSelected(self.entity)
                self.button.icon = list[i].icon
                self.listEntity.active = false
                if func then func(list[i]) end
            end            
        end
        
        function self.listEntity.update(entity, dt)
            local worldPos = self.entity.worldPosition
            entity.x = worldPos.x + 30
            entity.y = worldPos.y - 8
        end
        
        self.listEntity.active = false
    end
end

