function createButton(scn, txt, w, h, style)
    local p = scn.canvas:child()
    p.size = vec2(w, h)
    local btn = p:add(ui.button)
    btn.label.text = txt
    btn.label.shadow = color(32)
    btn.style = style
    return p
end

function createIconButton(scn, spr, w, h, style, padding)
    padding = padding or 0
    
    local p = scn.canvas:child()
    p.size = vec2(w, h)
    local btn = p:add(ui.button)
    btn.label.text = ""
    btn.style = style
    
    local icon = p:child("icon")
    local scale = 1
    
    local x, y, width, height = spr:rect()
    
    if width < height then
        scale = w / width
    else
        scale = h / height
    end
    
    icon.size = vec2((width * scale) - padding, (height * scale) - padding)
    icon:anchor(CENTER, CENTER)
    icon.sprite = spr
    
    return p
end


function buttonStyle(assetBase, name)
    local normal = image.read(assetBase[name])
    normal.smooth = false
    
    local pressed = image.read(assetBase[name..'_pressed'])
    pressed.smooth = false
    
    local selected = image.read(assetBase.grey)
    
    return
    {
        normal = { sprite = normal.slice:patch(5)},
        pressed = { sprite = pressed.slice:patch(5)},
        selected = { sprite = selected.slice:patch(5)}
    }
end

function loadUI()
    local sprites = asset.builtin.PixelUI
    local colored = sprites.Colored
    
    local resources = {}
    resources.blue = buttonStyle(colored, "blue")
    resources.green = buttonStyle(colored, "green")
    resources.grey = buttonStyle(colored, "grey")
    resources.red = buttonStyle(colored, "red")
    resources.yellow = buttonStyle(colored, "yellow")
    resources.space = image.read(sprites.space).slice:patch(5)
    
    local leftRounded = image(128, 128)
    context.push(leftRounded)
    background(0, 0, 0, 0)
    style.push().noStroke()
    style.fill(200 - 25)
    rect(0, 5, 128, 128 - 10, 0, 10, 10, 0)
    style.fill(255 - 25)
    rect(0, 5, 128 - 10, 128 - 10, 0, 10, 10, 0)
    style.pop()
    context.pop()
    
    local leftRoundedPressed = image(128, 128)
    context.push(leftRoundedPressed)
    background(0, 0, 0, 0)
    style.push().noStroke()
    style.fill(200)
    rect(0, 5, 128, 128 - 10, 0, 10, 10, 0)
    style.fill(255)
    rect(0, 5, 128 - 5, 128 - 10, 0, 10, 10, 0)
    style.pop()
    context.pop()
    
    local leftRoundedSelected = image(128, 128)
    context.push(leftRoundedSelected)
    background(0, 0, 0, 0)
    style.push().noStroke()
    style.fill(200 - 50)
    rect(0, 5, 128, 128 - 10, 0, 10, 10, 0)
    style.fill(255 - 50)
    rect(0, 5, 128 - 5, 128 - 10, 0, 10, 10, 0)
    style.pop()
    context.pop()
    
    resources.leftRounded =
    {
        image = leftRounded,
        imagePressed = leftRoundedPressed,
        imageSelected = leftRoundedSelected,
        normal = { sprite = leftRounded.slice:patch(15)},
        pressed = { sprite = leftRoundedPressed.slice:patch(15)},
        selected = { sprite = leftRoundedSelected.slice:patch(15)}
    }
    
    resources.topRounded =
    {
        normal = { sprite = leftRounded.slice:patch(15):rotate(-1)},
        pressed = { sprite = leftRounded.slice:patch(15):rotate(-1)},
        selected = { sprite = leftRounded.slice:patch(15):rotate(-1)}
    }
    
    resources.rightRounded =
    {
        normal = { sprite = leftRounded.slice:patch(15):rotate(-2)},
        pressed = { sprite = leftRounded.slice:patch(15):rotate(-2)},
        selected = { sprite = leftRounded.slice:patch(15):rotate(-2)}
    }
    
    return resources
end
